@@ -1,10 +1,11 @@
 package Test::Mojo::Role::SubmitForm;
 
 use Mojo::Base -base;
+use List::AllUtils qw/uniq/;
 use Role::Tiny;
 use Carp qw/croak/;
 
-our $VERSION = '1.001004'; # VERSION
+our $VERSION = '1.001005'; # VERSION
 
 sub click_ok {
     my ( $self, $selector, $extra_params ) = @_;
@@ -13,12 +14,12 @@ sub click_ok {
     my $el = $self->tx->res->dom->at($selector)
         or croak "Did not find element matching selector $selector";
     unless ( $el->tag eq 'form' ) {
-        if ( $el->attr('type') eq 'image' ) {
-            $extra_params->{ $el->attr('name') . '.x' } = 1;
-            $extra_params->{ $el->attr('name') . '.y' } = 1;
+        if ( $el->{type} eq 'image' ) {
+            $extra_params->{ $el->{name} . '.x' } = 1;
+            $extra_params->{ $el->{name} . '.y' } = 1;
         }
         else {
-            $extra_params->{ $el->attr('name') } = $el->attr('value');
+            $extra_params->{ $el->{name} } = $el->val;
         }
 
         $el = $el->ancestors('form')->first;
@@ -49,25 +50,31 @@ sub click_ok {
     }
 
     my $tx = $self->ua->build_tx(
-        $el->attr('method')||'GET' => $el->attr('action')
+        $el->{method}||'GET' => $el->{action}
             => form => \%form,
     );
 
     $self->request_ok( $tx );
-
-    $self;
 }
 
 sub _get_controls {
-    my ( $self, $el ) = @_;
+    my ( $self, $form ) = @_;
 
-    map +( $_->attr('name') => $_->val ),
-        $el->find(
-            'input:not([type=button]):not([type=submit]):not([type=image])'
-            . ':not([type=checkbox]):not([type=radio]),'
-            . '[type=checkbox][checked], [type=radio][checked],'
-            . 'select, textarea'
-        )->each;
+    my @els = $form->find(
+        'input:not([type=button]):not([type=submit]):not([type=image])'
+        . ':not([type=checkbox]):not([type=radio]),'
+        . '[type=checkbox]:checked, [type=radio]:checked,'
+        . 'select, textarea'
+    )->each;
+
+    my %controls;
+    for ( @els ) {
+        defined( my $val = $_->val ) or next;
+        push @{ $controls{$_->{name}} }, ref $val ? @$val : $val;
+    }
+    $#$_ or $_= $_->[0] for values %controls; # chage 1-el arrayrefs to strings
+
+    return %controls;
 }
 
 q|
